var __create = Object.create;
var __defProp = Object.defineProperty;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __markAsModule = (target) => __defProp(target, "__esModule", {value: true});
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, {get: all[name], enumerable: true});
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, {get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable});
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? {get: () => module2.default, enumerable: true} : {value: module2, enumerable: true})), module2);
};
var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};

// src/index.ts
__markAsModule(exports);
__export(exports, {
  bundle: () => bundle,
  handleError: () => handleError
});

// src/manifests/NodePackageManifest.ts
var path = __toModule(require("path"));

// src/utils/fs.ts
var import_fs = __toModule(require("fs"));

// src/utils/StandardError.ts
var StandardError = class extends Error {
  constructor() {
    super(...arguments);
    this.name = "StandardError";
  }
};

// src/utils/fs.ts
var findUp = require("find-up");
async function findParentFile(cwd, name) {
  const filePath = await findUp(name, {cwd, type: "file"});
  if (!filePath) {
    throw new StandardError(`Failed to find ${name} close to ${cwd}`);
  }
  return filePath;
}
async function readJSON(filePath) {
  try {
    const json = await import_fs.promises.readFile(filePath, "utf-8");
    return JSON.parse(json);
  } catch {
    throw new StandardError(`Failed to parse ${filePath} to json`);
  }
}

// src/utils/Logger.ts
var debug = require("debug");
function createLogger(...namespace) {
  const logger = debug(["rapidbundle", ...namespace].join(":"));
  logger.log = console.log.bind(console);
  return logger;
}

// src/manifests/NodePackageManifest.ts
var semver = require("semver");
var log = createLogger("manifest", "node-package");
var NodePackageManifest = class {
  static parsePkg(pkgPath, pkg) {
    var _a;
    const pkgDir = path.dirname(pkgPath);
    const manifest = new NodePackageManifest();
    if (typeof pkg.main == "string") {
      manifest.nodeEntry = path.join(pkgDir, pkg.main);
    }
    if (typeof ((_a = pkg.engines) == null ? void 0 : _a.node) == "string") {
      const minVersion = semver.minVersion(pkg.engines.node);
      if (!minVersion) {
        throw new StandardError("Invalid manifest: failed to parse `engines.version`");
      }
      manifest.nodeTarget = minVersion.version;
    }
    if (typeof pkg.dependencies == "object") {
      manifest.nodeDependencies = Object.keys(pkg.dependencies);
    }
    return manifest;
  }
  static async readPkg(pkgPath) {
    log("reading file from: %s", pkgPath);
    const pkg = await readJSON(pkgPath);
    return this.parsePkg(pkgPath, pkg);
  }
  static async findPkg(input) {
    log("reading package from: %s", input);
    if (!path.isAbsolute(input)) {
      input = path.resolve(input);
      log("normalize to absolute path: %s", input);
    }
    let pkgPath = input;
    if (path.basename(input) !== "package.json") {
      log("looking for the closest directory to: %s", input);
      pkgPath = await findParentFile(input, "package.json");
    }
    return this.readPkg(pkgPath);
  }
};

// src/tasks/BuildNodeTask.ts
var import_esbuild = __toModule(require("esbuild"));
var path2 = __toModule(require("path"));

// src/tasks/BuildTask.ts
var _name, _logger, _options;
var BuildTask = class {
  constructor(name, options) {
    _name.set(this, void 0);
    _logger.set(this, void 0);
    _options.set(this, void 0);
    __privateSet(this, _name, name);
    __privateSet(this, _options, options);
    __privateSet(this, _logger, createLogger("build", "task", name));
  }
  get cwd() {
    return __privateGet(this, _options).cwd;
  }
  log(format, ...args) {
    __privateGet(this, _logger).call(this, format, ...args);
  }
};
_name = new WeakMap();
_logger = new WeakMap();
_options = new WeakMap();

// src/tasks/BuildNodeTask.ts
var _manifest;
var BuildNodeTask = class extends BuildTask {
  constructor(_a) {
    var _b = _a, {manifest} = _b, options = __objRest(_b, ["manifest"]);
    super("node", options);
    _manifest.set(this, void 0);
    __privateSet(this, _manifest, manifest);
  }
  async run() {
    const {
      nodeEntry,
      nodeTarget = "12",
      nodeDependencies = []
    } = __privateGet(this, _manifest);
    if (!nodeEntry) {
      this.log("skipping build, reason: empty node entry");
      return;
    }
    this.log("building with options: %o", {nodeEntry, nodeTarget});
    await (0, import_esbuild.build)({
      bundle: true,
      keepNames: true,
      platform: "node",
      outfile: nodeEntry,
      target: `node${nodeTarget}`,
      external: nodeDependencies,
      entryPoints: [path2.join(this.cwd, "src")]
    });
  }
};
_manifest = new WeakMap();

// src/index.ts
async function bundle(cwd) {
  const manifest = await NodePackageManifest.findPkg(cwd);
  const tasks = [new BuildNodeTask({cwd, manifest})];
  for (const task of tasks) {
    await task.run();
  }
}
function handleError(error) {
  if (error instanceof StandardError) {
    console.error(error.message);
  } else {
    console.error(error);
  }
  process.exit(1);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  bundle,
  handleError
});
